from chattr_config.monitoring.service.monitor import Monitor
from typing import List, Union
import logging
import time

logger = logging.getLogger('django')


class MockMonitor(Monitor):

    name = 'MockMonitor'

    def __init__(self):
        super().__init__()

    def increment_metric(self, metric: str, tags: List[str]) -> None:
        pass

    def timer(self, metric: str, tags: List[str]) -> object:
        class TimedContextManagerDecorator(object):
            def __enter__(self):
                self.start = time.time()

            def __exit__(self, type, value, traceback):
                end = time.time()
                logger.info(f'Metric: {metric} with tags: {tags} took {end-self.start} secs')

        return TimedContextManagerDecorator()

    def send_metric(self, metric: str, value: Union[int, float], tags: List[str]) -> None:
        pass



import functools
import logging
import os
import time

logger = logging.getLogger(__name__)


def log_on_exception(_func=None, *, acceptable_exceptions=Exception):
    """
    ---------------------------------------------------------------------------------------------
    Decorator to log exception but not fail (default: Exception)
    ---------------------------------------------------------------------------------------------

    NOTE: Use with caution! Specified Exceptions will be eaten and None will be returned, might
          lead to unexpected situations on the 'decorated function' caller.
    Usage:

        @log_on_exception
        def get_data(*args, **kwargs):
            client.get_required_data(arg1)

    Examples:

        1. Log and return if Exception encountered (Default):
        @log_on_exception

        2. Log and return if IndexError encountered (Default):
        @log_on_exception(acceptable_exceptions=IndexError)

    """

    def decorator(_func):
        @functools.wraps(_func)
        def wrapper(*args, **kwargs):
            try:
                return _func(*args, **kwargs)
            except acceptable_exceptions:
                logger.exception(f'Exception occurred while running {_func.__name__}. Returning None to caller!')
            return None

        return wrapper

    return decorator if _func is None else decorator(_func)


def retry_on_exception(_func=None, *, num_retries=3, retry_exceptions=Exception, exception_handler=None, delay=0):
    """
    ---------------------------------------------------------------------------------------------
    Retry the function num_retries (default: 3) on encountering an exception (default: Exception)
    ---------------------------------------------------------------------------------------------

    Usage:

        @retry_on_exception
        def get_data(*args, **kwargs):
            client.get_required_data(arg1)

    Examples:

        1. Retry 3 times if Exception encountered (Default):
        @retry_on_exception

        2. Retry 1 time if IndexError encountered:
        @retry_on_exception(retry_exceptions=IndexError, num_retries=1)

        3. Retry 2 times if IndexError or ValueError encountered:
        @retry_on_exception(retry_exceptions=(IndexError, ValueError), num_retries=2)

        4. Retry 3 times if IndexError or ValueError with Linear delay of 5 seconds:
        @retry_on_exception(retry_exceptions=(IndexError, ValueError), num_retries=3, delay=5)
        First retry will be after 5 secs, second after (5+)10 secs and third after (5+10+)15 secs

    """

    def decorator_retry(_func):
        @functools.wraps(_func)
        def wrapper_retry(*args, **kwargs):
            for i in range(num_retries):
                try:
                    return _func(*args, **kwargs)
                except retry_exceptions:
                    logger.exception(
                        f'Exception occurred while running {_func.__name__} ' f'on attempt {i+1} out of {num_retries}.'
                    )
                    if type(delay) == int and delay > 0:
                        if exception_handler:
                            logger.warning(f'Executing {exception_handler.__name__}.')
                            exception_handler()
                        linear_delay = (i + 1) * delay
                        time.sleep(linear_delay)
                        logger.info(f'Retrying {_func.__name__} {i+1} time after {linear_delay} seconds.')
            return _func(*args, **kwargs)

        return wrapper_retry

    return decorator_retry if _func is None else decorator_retry(_func)


def function_not_implemented(error_message_prefix=None, error_message_suffix=None):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            error_message = f'{func.__name__} not implemented!'

            if error_message_prefix and type(error_message_prefix) is str:
                if error_message_prefix[-1] != ' ':
                    error_message = error_message_prefix + ' ' + error_message
                else:
                    error_message = error_message_prefix + error_message

            if error_message_suffix and type(error_message_suffix) is str:
                error_message += ' ' + error_message_suffix

            raise NotImplementedError(error_message)

        return wrapper

    return decorator


def are_we_in_test_mode():
    settings_module_string = os.environ.get("DJANGO_SETTINGS_MODULE", "local").split(".")[-1]
    return 'local' in settings_module_string or 'test' in settings_module_string


def skip_in_test_mode(f):
    def internal(*args, **kwargs):
        if not are_we_in_test_mode():
            return f(*args, **kwargs)
        return None

    return internal



import dataclasses
import logging
import threading as thread
import uuid
from typing import Any, ClassVar, Dict, Set, Tuple, cast, Optional

#from ddtrace.helpers import get_correlation_ids
from ddtrace.tracer import Tracer

from chattr_config.monitoring.utils import datadog_enabled


@dataclasses.dataclass
class Context:
    ASSIGNABLE_FIELDS: ClassVar[Set[str]] = {"brand_id", "branduser_id"}
    _context_map: ClassVar[Dict[int, Tuple[object, object]]] = {}

    version_id: str = None
    brand_id: int = None
    branduser_id: int = None
    trace_id: str = None  # Used by the APM for tracing purposes that would differ from our internal one.
    correlation_id: str = None
    request_id: str = None
    impersonator: Optional[str] = None

    def __post_init__(self):
        self.version_id = self.version_id or 'v1'
        self.correlation_id = self.correlation_id or str(uuid.uuid4())
        if not self.trace_id and datadog_enabled():
            self.tracer = Tracer()
            #self.trace_id, span_id = get_correlation_ids()
            self.trace_id = self.tracer.get_log_correlation_context().get('trace_id')
            self.span_id = self.tracer.get_log_correlation_context().get('span_id')
        self.request_id = self.request_id or str(uuid.uuid4())

    def to_dict(self):
        return self.__dict__

    @classmethod
    def acquire_from_dict(cls, context_dict: Dict[str, Any], set_context: bool = False):
        # request_id is not assigned because this is on the receiving side of
        # the context, and thus, we would want to create a different request_id
        if context_dict:
            context = cls(
                version_id=context_dict.get("version_id"),
                correlation_id=context_dict.get("correlation_id"),
                brand_id=context_dict.get("brand_id"),
                trace_id=context_dict.get("trace_id"),
                branduser_id=context_dict.get("branduser_id"),
                impersonator=context_dict.get('impersonator'),
            )
        else:
            logging.warning(f"Attempted to acquire Context from {context_dict} as dict. Creating new Context")
            context = cls()

        if set_context:
            cls._context_map[thread.get_ident()] = (None, context)

        return context

    @classmethod
    def acquire(cls, request=None, fields: Optional[Dict[str, str]] = None) -> "Context":
        fields = fields or {}
        entry: Tuple[object, Context] = cast(Tuple[object, Context], cls._context_map.get(thread.get_ident()))
        if not entry:
            entry = cls._make_entry(request)
        request, context = entry
        usable_keys = set(fields.keys()).intersection(cls.ASSIGNABLE_FIELDS)
        for key in usable_keys:
            setattr(context, key, fields[key])

        _add_jwt_claims(request, context)

        return context

    @classmethod
    def release(cls, request=None):
        entry = cls._context_map.get(thread.get_ident())
        if not entry:
            logging.warning("Context mismatch where the entry does not exist.")
            return

        del Context._context_map[thread.get_ident()]
        if request and hasattr(request, "context"):
            delattr(request, "context")

    @classmethod
    def _make_entry(cls, request=None):
        if request:
            ctx = _make_context(request)
            request.context = ctx
        else:
            ctx = Context()
        entry = cls._context_map[thread.get_ident()] = (request, ctx)
        return entry


def _make_context(request) -> Context:

    headers = request.headers
    # https://docs.datadoghq.com/synthetics/apm
    trace_id = headers.get("x-datadog-trace-id")
    # https://en.wikipedia.org/wiki/List_of_HTTP_header_fields
    # Generated by Chattr.io servers.
    correlation_id = headers.get("X-Correlation-ID")

    # Generated by clients but can be defaulted to a server rendered version.
    request_id = headers.get("X-Request-ID")
    return Context(trace_id=trace_id, correlation_id=correlation_id, request_id=request_id)


def _add_jwt_claims(request, context) -> None:
    if request and getattr(request, 'authed_user'):
        context.impersonator = context.impersonator or getattr(request.authed_user, 'impersonator')
        context.brand_id = context.brand_id or getattr(request.authed_user, 'brand_id')



from abc import ABC, abstractmethod
from typing import Union, List


class Monitor(ABC):
    name = 'AbstractMonitor'
    acceptable_tags = ['producer', 'consumer', 'topic', 'environment', 'brand_id', 'status', 'task', 'queue']

    def get_tags(self, **tags_dict: str) -> List[str]:
        return [f'{tag}:{tags_dict.get(tag)}' for tag in self.acceptable_tags]

    @abstractmethod
    def increment_metric(self, metric: str, tags: List[str]) -> None:
        pass

    @abstractmethod
    def timer(self, metric: str, tags: List[str]) -> object:
        pass

    @abstractmethod
    def send_metric(self, metric: str, value: Union[int, float], tags: List[str]) -> None:
        pass



from chattr_config.monitoring.service.monitor import Monitor
from datadog import statsd
from typing import Union, List
from chattr_config.decorators.decorators import log_on_exception


class DatadogMonitor(Monitor):

    name = 'Datadog'

    def __init__(self):
        super().__init__()
        self.statsd = statsd

    @log_on_exception
    def increment_metric(self, metric: str, tags: List[str]) -> None:
        self.statsd.increment(metric, tags=tags)

    @log_on_exception
    def timer(self, metric: str, tags: List[str]) -> object:
        return statsd.timed(metric, tags=tags)

    @log_on_exception
    def send_metric(self, metric: str, value: Union[int, float], tags: List[str]) -> None:
        statsd.histogram(metric, value, tags=tags)



from typing import Any

# bool() returns true on any defined value
# strtobool() can't handle a bool being passed in
# anything else is just a false.
def evalbool(value: Any) -> bool:
    return_value: bool = False
    if type(value) is bool:
        return_value = value

    if type(value) is str:
        if value.lower() in ('y', 'yes', 't', 'true', 'on', '1'):
            return_value = True

    if type(value) is int:
        if value == 1:
            return_value = True

    return return_value



from functools import lru_cache
import os
import socket


@lru_cache()
def get_ip_address():
    try:
        ip_address = socket.gethostbyname(socket.gethostname())
    except socket.gaierror:
        ip_address = '127.0.0.1'

    return ip_address


@lru_cache()
def get_fqdn():
    return socket.getfqdn()


@lru_cache()
def get_process_id():
    return os.getpid()


@lru_cache()
def get_machine_info():
    return {'ip_address': get_ip_address(), 'fully_qualified_domain_name': get_fqdn(), 'process_id': get_process_id()}



# This has nothing to do with custom settings per brand.  It doesn't have that much granularity.
# We can have something that the server can turn on and off for feature flags here.

# Load yaml file based upon 'env' variable.

# Direction is to pull from SSM to start as well.

# Pull in environment variables.

# Order importance:
#   - ENV variable; these should be extremely rare
#   - SSM variable; these are _only_ for secrets and things that could be more easily configurable.
#   - yaml config file that pulls in all variables that pivots it.

import os
import logging
from typing import Any, Dict, Iterable, List

from chattr_config.config_loader.borg import Borg
from chattr_config.config_loader.ssm_loader import SSMLoader
from chattr_config.config_loader.yaml_loader import YamlLoader

from .definitions import ConfigDefinitions


_logger = logging.getLogger(__name__)


class ConfigImpl(Borg):
    def __init__(self) -> None:
        """Constructor for configuration."""
        super().__init__()
        self.lookup_table: Dict[str, Any] = {}
        self.loaded = False

    def init(self, var_file: str, value_files: List[str], ssm_folder: str = None, ssm_client=None):
        """Loads up the files into yaml and validate that all values
            are correctly filled.

        Args:
            var_file (str): Variable declaration file that then will iterate
                and find the needed from configuration on load
            value_files (List[str]): Files that contain value that can be shared across yaml files
            ssm_folder (str): base path for secrets storage.
            ssm_client (botocore.client.SSM): boto3 ssm client object
        Returns:
            Errors on loading the yaml files.
        """
        validator = ConfigDefinitions(var_file)
        validator.load()
        config_data: Dict[str, Any] = {}
        # The last one in the list has the highest priority
        for file in value_files:
            file_data = YamlLoader.load_file(file)
            config_data.update(file_data)

        # SSM is highest priority except for environment variables
        if ssm_folder and os.environ.get('SSM_LOADER_DISABLED', '').lower() not in ['true', '1']:
            # Merge SSM value into the necessary dictionary after load.
            ssm_data = SSMLoader(prefix=ssm_folder, ssm_client=ssm_client)
            config_data = {**config_data, **ssm_data}

        config_data.update(self._load_environ_values(validator.keys()))

        # Ask ConfigDefinitions to validate and normalize the resultant dictionary.
        self.lookup_table, errors = validator.transform(config_data)
        if errors:
            raise Exception("Unable to load configurations.  Check logs for problems with config")

        self.loaded = True

    def _load_environ_values(self, keys: Iterable[str]) -> Dict[str, Any]:
        data: Dict[str, Any] = {}
        for key in keys:
            result = os.environ.get(key, None)
            if result is not None:
                data[key] = result
        return data

    '''
    We cast all values after we load the definitions file. Prefer using `get`
    over these individual methods. If the type is wrong, fix the definitions file.
    '''

    def int_value(self, name: str, default: int = -1) -> int:
        _logger.warning('`int_value` will be removed in a future release. Use `get` for retrieving config values.')
        return self._get_value(name, default, int, "int")

    def str_value(self, name: str, default: str = "") -> str:
        _logger.warning('`str_value` will be removed in a future release. Use `get` for retrieving config values.')
        return self._get_value(name, default, str, "str")

    def bool_value(self, name: str, default: bool = False) -> bool:
        _logger.warning('`bool_value` will be removed in a future release. Use `get` for retrieving config values.')
        return self._get_value(name, default, bool, "bool")

    def float_value(self, name: str, default: float = 0.0) -> float:
        _logger.warning('`float_value` will be removed in a future release. Use `get` for retrieving config values.')
        return self._get_value(name, default, float, "float")

    def _get_value(self, name: str, default: Any, field_type, field_type_name: str = "") -> Any:
        if not self.loaded:
            raise Exception("Not initialized")
        result = self.lookup_table.get(name)
        if result is None:
            result = default
        elif not isinstance(result, field_type):
            raise ValueError(f"{name} is not of type {field_type_name}")
        return result

    def get(self, name: str, default: Any = None) -> Any:
        if not self.loaded:
            raise Exception("Not initialized")
        result = self.lookup_table.get(name)
        if result is None:
            result = default
        return result


ConfigData = ConfigImpl()



__version__ = '0.1.0'
from chattr_config.config_loader.borg import Borg
from chattr_config.config_loader.config import loader
from chattr_config.config_loader.config.loader import ConfigData
from chattr_config.config_loader.ssm_loader import SSMLoader
from chattr_config.config_loader.yaml_loader import YamlLoader
from chattr_config.log_formatter.chattr_json_log_formatter import ChattrJSONFormatter
from chattr_config.monitoring.context import Context
from chattr_config.utils.system_utils import get_ip_address, get_fqdn, get_process_id, get_machine_info



import os

import pendulum

from chattr_config.monitoring.context import Context
from chattr_config.utils.system_utils import get_machine_info

from .base_json_log_formatter import JSONFormatter


class ChattrJSONFormatter(JSONFormatter):
    """
    Custom class to override the default behaviour of the JSONFormatter
    """

    def format(self, record):
        """
        the default behaviour of JSONFormatter is to cast everything into a string
        we avoid calling getMessage and leave the object as it is, this way our json messages can have nested dicts
        """
        message = record.getMessage()
        extra = self.extra_from_record(record)
        json_record = self.json_record(message, extra, record)
        mutated_record = self.mutate_json_record(json_record)
        # Backwards compatibility: Functions that overwrite this but don't
        # return a new value will return None because they modified the
        # argument passed in.
        if mutated_record is None:
            mutated_record = json_record
        mutated_record.update({'machine_info': get_machine_info()})
        mutated_record.update({'context': Context.acquire().to_dict()})
        return self.to_json(mutated_record)

    def json_record(self, message, extra, record):
        extra['level'] = record.levelname
        extra['module'] = record.name
        extra['time'] = pendulum.now()
        extra['message'] = message
        extra['func_name'] = f"{record.module}.{record.funcName}:{record.lineno}"

        if record.exc_info:
            extra['exec_info'] = self.formatException(record.exc_info)
        return extra

    def to_json(self, record):
        """Converts record dict to a JSON string.

        It makes best effort to serialize a record (represents an object as a string)
        instead of raising TypeError if json library supports default argument.
        Note, ujson doesn't support it.
        """
        return self.json_lib.dumps(record, default=_json_object_encoder)


def _json_object_encoder(obj):
    try:
        return obj.to_json()
    except AttributeError:
        return str(obj)



import json

import dataclasses
import logging
from typing import Any, Callable, Dict, List, Set, Tuple, Union, Optional

from chattr_config.config_loader.config.conversion import evalbool
from chattr_config.config_loader.yaml_loader import YamlLoader


@dataclasses.dataclass(frozen=True)
class DefinitionsData:
    name: str
    value: Union[int, float, bool, str]
    type_name: str
    cast_method: Callable
    secret: bool = False
    required: bool = False


cast_methods = {
    "bool": evalbool,
    "float": float,
    "int": int,
    "string": str,
    'json': json.loads,
}


class ConfigDefinitions:
    def __init__(self, var_file: str) -> None:
        """Constructor for definitions file.

        Args:
            var_file (str): Variable declaration file that then will iterate
                and find the needed from configuration on load
        """
        self.var_file = var_file
        self.definition: Dict[str, DefinitionsData] = {}

    def load(self):
        """Loads up the files into yaml and validate that all values
            are correctly filled.

        Returns:
            Errors on loading the yaml files.
        """
        data = YamlLoader.load_file(self.var_file)
        self.definitions = self._parse_definitions(data)
        self.required_items = {key for key, value in self.definitions.items() if value.required is True}
        self.defaults = {key: value for key, value in self.definition.items() if value.value is not None}

    def transform(self, input_data: Dict[str, Any]) -> Tuple[Dict[str, Any], List[Exception]]:
        """Iterates through the dictionary of key:value pairs
            and casts to the appropriate type.
        Args:
            input_data (Dict[str, Any]): Input data with each key being an entry in the definitions
                that defines what it uses.

        Returns:
            List[ValueError]: [description]
        """
        errors: List[Exception] = []
        result: Dict[str, Any] = {}
        for key, definition in self.definitions.items():
            value = input_data.get(key, definition.value)
            try:
                result[key] = cast_methods[definition.type_name](value) if value is not None else None
            except ValueError as e:
                logging.error(f"Cannot cast {key}:{value} to defined type {definition.type_name}")
                errors.append(e)

        # add input the dictionary any defaults that do not exist in the result
        unfilled_keys = self.defaults.keys() - result.keys()
        for key in unfilled_keys:
            result[key] = self.defaults[key]

        # check if all required items are accounted for
        has_default_values = {k: v for k, v in result.items() if v is not None}
        unknown_keys: Set[str] = self.required_items - set(list(has_default_values.keys()))
        if unknown_keys:
            logging.error(f"Unaccounted for required keys {unknown_keys}")
            raise ValueError(f"Missing values for {sorted(unknown_keys)}")

        return result, errors

    def keys(self) -> Set[str]:
        return set(self.definitions.keys())

    def _parse_definitions(self, config_data: Dict[str, Any]) -> Dict[str, DefinitionsData]:
        output: Dict[str, DefinitionsData] = {}
        errors: List[Exception] = []
        for key, value in config_data.items():
            try:
                data = value.get("value")
                type_data = value.pop("type")
                value["type_name"] = type_data
                cast_method = cast_methods[type_data]
                if data is None:
                    value["value"] = None
                value["cast_method"] = cast_method
                output[key] = DefinitionsData(name=key, **value)
            except Exception as e:
                logging.error(f"Errors found parsing: {key}:{value}")
                print(e)
                errors.append(e)
        if errors:
            logging.error("Check logs for failed definitions loader")
            raise Exception("Unable to load definitions file")

        return output



import logging
import os
from chattr_config.monitoring.service.datadog import DatadogMonitor
from chattr_config.monitoring.service.monitor import Monitor
from chattr_config.monitoring.service.mock_monitor import MockMonitor
from singleton_decorator import singleton
from typing import Union, List


@singleton
class MonitoringService(object):
    def __init__(self, monitor=None, logger=logging.getLogger('django')):
        self.monitor = monitor or MockMonitor()
        self.logger = logger

    def get_monitor_name(self):
        return self.monitor.name

    def get_tags(self, **tags_dict) -> List[str]:
        return self.monitor.get_tags(**tags_dict)

    def increment_metric(self, metric: str, tags: List[str]) -> None:
        self.monitor.increment_metric(metric, tags)

    def timer(self, metric: str, tags: List[str]):
        return self.monitor.timer(metric, tags)

    def send_metric(self, metric: str, value: Union[int, float], tags: List[str]):
        return self.monitor.send_metric(metric, value, tags)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chattr_config',
 'chattr_config.config_loader',
 'chattr_config.config_loader.config',
 'chattr_config.decorators',
 'chattr_config.log_formatter',
 'chattr_config.monitoring',
 'chattr_config.monitoring.service',
 'chattr_config.utils']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'boto3>=1.9.201',
 'click>=8.1.3,<8.2.0',
 'datadog==0.44.0',
 'ddtrace>=1.9.2,<2.0.0',
 'pendulum>=2.1.2,<3.0.0',
 'singleton-decorator>=1.0.0,<2.0.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.8,<0.9']}

setup_kwargs = {
    'name': 'chattr-config',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'Chattr',
    'author_email': 'chattra23@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)



import logging
import os
import random
import time
from typing import Any

from boto3.session import Session

_logger = logging.getLogger(__name__)

MAX_FAILED_ATTEMPTS = 3


class SSMLoader:
    """
    Provide a dictionary-like interface to access AWS SSM Parameter Store
    """

    def __init__(self, prefix=None, ssm_client=None):
        self._prefix = (prefix or '').rstrip('/') + '/'
        self._client = ssm_client or self._make_client()
        self._keys = {}
        self._sub_stores = {}

    def _make_client(self):
        aws_region = os.environ.get('AWS_DEFAULT_REGION') or 'us-west-2'
        return Session(region_name=aws_region).client('ssm')

    def _get_all_dataparams(self, base_path: str):
        """root of all parameters that is to be pulled out
            (dev/stg/prf/prd)/(config|microservicename)/settings

        Args:
            base_path (str): base of services code.
        """
        data = []
        query = {'Path': base_path, 'Recursive': True, 'WithDecryption': True}
        done = False
        while not done:
            result = self._client.get_parameters_by_path(**query)
            meta = result['ResponseMetadata']
            if meta['HTTPStatusCode'] == 429:
                self._back_off(meta['RetryAttempts'])
            parameters = [(item['Name'], item['Value']) for item in result['Parameters']]
            data.extend(parameters)
            query['NextToken'] = next_token = result.get('NextToken')
            if len(parameters) < 1 or not next_token:
                done = True

        start_at = len(base_path)
        return {item[0][start_at:].upper(): item[1] for item in data}

    def _back_off(self, failed_attempts: int):
        if failed_attempts > MAX_FAILED_ATTEMPTS:
            raise Exception(f'SSM Load Failed with too many failed attempts {failed_attempts}')

        jitter = random.randint(0, 2000)
        sleep_time = ((2 << failed_attempts) * 1000 + jitter) / 1000
        _logger.warning(f'Backing off {sleep_time} with failed_attempts {failed_attempts}')
        time.sleep(sleep_time)

    def get(self, name: str, **kwargs) -> Any:
        if not self._keys:
            self.refresh()

        if name not in self._keys:
            if 'default' in kwargs:
                return kwargs['default']
            raise KeyError(name)
        else:
            return self._get_value(name)

    def refresh(self):
        self._keys = {}
        self._keys = self._get_all_dataparams(self._prefix)

    def keys(self) -> list:
        if not self._keys:
            self.refresh()
        return self._keys.keys()

    def _get_value(self, name) -> Any:
        return self._keys[name]

    def __contains__(self, name: str) -> bool:
        try:
            self.get(name)
        except (AttributeError, KeyError):
            return False
        else:
            return True

    def __getitem__(self, name: str) -> Any:
        return self.get(name)

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, name):
        raise NotImplementedError

    def __repr__(self) -> str:
        return f'ParameterStore[{self._prefix}]'



from typing import Any, Dict

import os

import yaml


class YamlLoader:
    @staticmethod
    def load_file(file_path: str) -> Dict[str, Any]:
        if not os.path.exists(file_path):
            return {}

        with open(file_path, "r") as f:
            config = yaml.load(f, Loader=yaml.Loader)
        return config or {}
